@@ -27,7 +27,7 @@ module Database.HaskellDB.Query (
 	     , (.*.) , (./.), (.%.), (.+.), (.-.), (.++.)
              , (<<), (<<-)
 	      -- * Function declarations
-	     , project, restrict, table
+	     , project, restrict, table, unique
 	     , union, intersect, divide, minus
 	     , _not, like, _in, cat, _length
 	     , isNull, notNull
@@ -116,9 +116,9 @@ attributeName (Attr name) = name
 class ExprC e where
     -- | Get the underlying untyped 'PrimExpr'.
     primExpr :: e a -> PrimExpr
-instance ExprC Expr where primExpr (Expr e) = e
-instance ExprC ExprAggr where primExpr (ExprAggr e) = e
-instance ExprC ExprDefault where primExpr (ExprDefault e) = e
+instance ExprC Expr where primExpr ~(Expr e) = e
+instance ExprC ExprAggr where primExpr ~(ExprAggr e) = e
+instance ExprC ExprDefault where primExpr ~(ExprDefault e) = e
 
 -- | Class of expressions that can be used with 'insert'.
 class ExprC e => InsertExpr e
@@ -199,6 +199,29 @@ project r
 restrict :: Expr Bool -> Query ()
 restrict (Expr primExpr) = updatePrimQuery_ (Restrict primExpr)
 
+-- | Restricts the relation given to only return unique records. Upshot
+-- is all projected attributes will be 'grouped'
+unique :: Query ()
+unique = Query (\(i, primQ) ->
+    -- Add all non-aggregate expressions in the query
+    -- to a groupby association list. This list holds the name
+    -- of the expression and the expression itself. Those expressions
+    -- will later by added to the groupby list in the SqlSelect built.
+    case findNonAggr primQ of
+      [] -> ((), (i + 1, primQ)) -- No non-aggregate expressions - no-op.
+      newCols -> ((), (i + 1, Group newCols primQ)))
+  where
+    -- Find all non-aggregate expressions.
+    findNonAggr :: PrimQuery -> Assoc
+    findNonAggr (Project cols q) = filter (not . isAggregate . snd) cols
+    findNonAggr (Restrict _ q) = findNonAggr q
+    findNonAggr (Binary _ q1 q2) = findNonAggr q1 ++ findNonAggr q2
+    findNonAggr (BaseTable tblName cols) = zip cols (map AttrExpr cols)
+    findNonAggr (Special _ q) = findNonAggr q
+    -- Group and Empty are no-ops
+    findNonAggr (Group _ _) = []
+    findNonAggr Empty  = []
+  
 -----------------------------------------------------------
 -- Binary operations
 -----------------------------------------------------------
@@ -470,7 +493,7 @@ instance ConstantRecord RecNil RecNil where
 
 instance (ShowConstant a, ConstantRecord r cr) 
     => ConstantRecord (RecCons f a r) (RecCons f (Expr a) cr) where
-    constantRecord (RecCons x rs) = RecCons (constant x) (constantRecord rs)
+    constantRecord ~(RecCons x rs) = RecCons (constant x) (constantRecord rs)
 
 -----------------------------------------------------------
 -- Aggregate operators
@@ -630,10 +653,10 @@ class ToPrimExprs r where
     toPrimExprs :: r -> [PrimExpr]
 
 instance ToPrimExprs RecNil where
-    toPrimExprs RecNil = []
+    toPrimExprs ~RecNil = []
 
 instance (ExprC e, ToPrimExprs r) => ToPrimExprs (RecCons l (e a) r) where
-    toPrimExprs (RecCons e r) = primExpr e : toPrimExprs r
+    toPrimExprs ~(RecCons e r) = primExpr e : toPrimExprs r
 
 {-
 
